@@ -11,29 +11,48 @@
 # You should have received a copy of the GNU General Public License along with django-ca. If not, see
 # <http://www.gnu.org/licenses/>.
 
+"""Collection of exception classes for ACMEv2."""
+
+from typing import Any
+from typing import Type
+
 from .responses import AcmeResponseBadCSR
 from .responses import AcmeResponseError
+from .responses import AcmeResponseForbidden
 from .responses import AcmeResponseMalformed
 from .responses import AcmeResponseUnauthorized
 
 
 class AcmeException(Exception):
-    response = AcmeResponseError
+    """Base class for all ACME exceptions."""
 
-    def __init__(self, *args, **kwargs):
+    response: Type[AcmeResponseError] = AcmeResponseError
+
+    def __init__(self, *args: Any, **kwargs: Any) -> None:
         super().__init__(*args)
         self.kwargs = kwargs
 
-    def get_response(self):
+    def get_response(self) -> AcmeResponseError:
+        """Get the corresponding ACME response class."""
         return self.response(*self.args, **self.kwargs)
 
 
 class AcmeMalformed(AcmeException):
-    response = AcmeResponseMalformed
+    """Exception when the request was malformed."""
+
+    response: Type[AcmeResponseMalformed] = AcmeResponseMalformed
 
 
 class AcmeUnauthorized(AcmeException):
-    response = AcmeResponseUnauthorized
+    """Exception when the request is unauthorized."""
+
+    response = AcmeResponseUnauthorized  # 401
+
+
+class AcmeForbidden(AcmeException):
+    """Exception when the request is unauthorized."""
+
+    response = AcmeResponseForbidden  # 403
 
 
 class AcmeBadCSR(AcmeException):
