@@ -24,6 +24,7 @@ import logging
 import secrets
 import typing
 from datetime import datetime
+from datetime import timezone as tz
 from http import HTTPStatus
 from typing import Dict
 from typing import Generic
@@ -38,13 +39,13 @@ from typing import cast
 
 import acme.jws
 import josepy as jose
-import pytz
 from acme import messages
 
 from cryptography import x509
 from cryptography.hazmat.primitives import hashes
 from cryptography.hazmat.primitives import serialization
 from cryptography.hazmat.primitives.serialization import Encoding
+from cryptography.x509.oid import NameOID
 
 from django.conf import settings
 from django.core.cache import cache
@@ -56,7 +57,6 @@ from django.http import HttpRequest
 from django.http import HttpResponse
 from django.http import JsonResponse
 from django.urls import reverse
-from django.utils import timezone
 from django.utils.decorators import method_decorator
 from django.views.decorators.csrf import csrf_exempt
 from django.views.generic.base import View
@@ -69,10 +69,11 @@ from ..models import AcmeCertificate
 from ..models import AcmeChallenge
 from ..models import AcmeOrder
 from ..models import CertificateAuthority
-from ..subject import Subject
 from ..tasks import acme_issue_certificate
 from ..tasks import acme_validate_challenge
 from ..tasks import run_task
+from ..utils import check_name
+from ..utils import make_naive
 from ..utils import validate_email
 from .errors import AcmeBadCSR
 from .errors import AcmeException
@@ -106,6 +107,50 @@ if typing.TYPE_CHECKING:
     from django.http.response import HttpResponseBase
 
 
+class ContactValidationMixin:
+    """Mixin for validating contact information."""
+
+    def validate_contacts(self, message: messages.Registration) -> None:
+        """Validate the contact information for this message."""
+
+        for contact in message.contact:
+            if contact.startswith(messages.Registration.email_prefix):
+                addr = contact[len(messages.Registration.email_prefix) :]
+
+                # RFC 8555, section 7.3
+                #
+                #   Clients MUST NOT provide a "mailto" URL in the "contact" field that contains "hfields"
+                #   [RFC6068] or more than one "addr-spec" in the "to" component.
+
+                # We rule out quoted local address fields, otherwise it's extremely hard to validate
+                # email addresses.
+                if addr.startswith('"'):
+                    raise AcmeMalformed("invalidContact", "Quoted local part in email is not allowed.")
+
+                # Since the local part is not quoted, it cannot contain a ',' either, so a ',' means there
+                # is more than one "addr-spec" in the "to" component. (see RFC 8555 quote above)
+                if "," in addr:
+                    raise AcmeMalformed("invalidContact", "More than one addr-spec is not allowed.")
+
+                # Validate that there are no hfields in the address.
+                # NOTE: ',' appears to be valid in the local part according to RFC 5322
+                _local, domain = addr.split("@", 1)
+                if "?" in domain:
+                    raise AcmeMalformed("invalidContact", f"{domain}: hfields are not allowed.")
+
+                # Finally, verify that we're getting at least a valid domain.
+                try:
+                    validate_email(addr)
+                except ValueError as ex:
+                    raise AcmeMalformed("invalidContact", f"{domain}: Not a valid email address.") from ex
+            else:
+                # RFC 8555, section 7.3
+                #
+                #   If the server rejects a contact URL for using an unsupported scheme, it MUST raise an
+                #   error of type "unsupportedContact", ...
+                raise AcmeMalformed("unsupportedContact", f"{contact}: Unsupported address scheme.")
+
+
 class AcmeDirectory(View):
     """
     `Equivalent LE URL <https://acme-v02.api.letsencrypt.org/directory>`__
@@ -315,6 +360,11 @@ class AcmeBaseView(AcmeGetNonceViewMixin, View, metaclass=abc.ABCMeta):
             except AcmeAccount.DoesNotExist:
                 return AcmeResponseUnauthorized(message="Account not found.")
             if account.usable is False:
+                # RFC 855, 7.3.6:
+                #
+                #   If a server receives a POST or POST-as-GET from a deactivated account, it MUST return an
+                #   error response with status code 401 (Unauthorized) and type
+                #   "urn:ietf:params:acme:error:unauthorized".
                 return AcmeResponseUnauthorized(message="Account not usable.")
             # self.prepared['thumbprint'] = account.thumbprint
             # self.prepared['pem'] = account.pem
@@ -398,6 +448,7 @@ class AcmeMessageBaseView(AcmeBaseView, Generic[MessageTypeVar], metaclass=abc.A
     def process_acme_request(self, slug: Optional[str]) -> AcmeResponse:
         try:
             message = self.message_cls.json_loads(self.jws.payload)
+            log.debug("ACME message: %s", message)
         except jose.DeserializationError as e:
             return AcmeResponseMalformedPayload(message=", ".join(e.args))
 
@@ -444,7 +495,7 @@ class AcmeNewNonceView(AcmeGetNonceViewMixin, View):
         return HttpResponse(status=HTTPStatus.NO_CONTENT)  # 204, unlike HEAD, which has 200
 
 
-class AcmeNewAccountView(AcmeMessageBaseView[messages.Registration]):
+class AcmeNewAccountView(ContactValidationMixin, AcmeMessageBaseView[messages.Registration]):
     """Implements endpoint for creating a new account, that is ``/acme/new-account``.
 
     This view is called when the ACME client tries to register a new account.
@@ -455,46 +506,6 @@ class AcmeNewAccountView(AcmeMessageBaseView[messages.Registration]):
     message_cls = messages.Registration
     requires_key = True
 
-    def validate_contacts(self, message: messages.Registration) -> None:
-        """Validate the contact information for this message."""
-
-        for contact in message.contact:
-            if contact.startswith(messages.Registration.email_prefix):
-                addr = contact[len(messages.Registration.email_prefix) :]
-
-                # RFC 8555, section 7.3
-                #
-                #   Clients MUST NOT provide a "mailto" URL in the "contact" field that contains "hfields"
-                #   [RFC6068] or more than one "addr-spec" in the "to" component.
-
-                # We rule out quoted local address fields, otherwise it's extremely hard to validate
-                # email addresses.
-                if addr.startswith('"'):
-                    raise AcmeMalformed("invalidContact", "Quoted local part in email is not allowed.")
-
-                # Since the local part is not quoted, it cannot contain a ',' either, so a ',' means there
-                # is more than one "addr-spec" in the "to" component. (see RFC 8555 quote above)
-                if "," in addr:
-                    raise AcmeMalformed("invalidContact", "More than one addr-spec is not allowed.")
-
-                # Validate that there are no hfields in the address.
-                # NOTE: ',' appears to be valid in the local part according to RFC 5322
-                _local, domain = addr.split("@", 1)
-                if "?" in domain:
-                    raise AcmeMalformed("invalidContact", f"{domain}: hfields are not allowed.")
-
-                # Finally, verify that we're getting at least a valid domain.
-                try:
-                    validate_email(addr)
-                except ValueError as ex:
-                    raise AcmeMalformed("invalidContact", f"{domain}: Not a valid email address.") from ex
-            else:
-                # RFC 8555, section 7.3
-                #
-                #   If the server rejects a contact URL for using an unsupported scheme, it MUST raise an
-                #   error of type "unsupportedContact", ...
-                raise AcmeMalformed("unsupportedContact", f"{contact}: Unsupported address scheme.")
-
     # TODO: possible to make slug non-optional?
     def acme_request(  # pylint: disable=unused-argument
         self, message: messages.Registration, slug: Optional[str]
@@ -578,16 +589,46 @@ class AcmeNewAccountView(AcmeMessageBaseView[messages.Registration]):
         return AcmeResponseAccountCreated(self.request, account)
 
 
-class AcmeAccountView(AcmeBaseView):  # pylint: disable=abstract-method
+class AcmeAccountView(
+    ContactValidationMixin, AcmeMessageBaseView[messages.Registration]
+):  # pylint: disable=abstract-method
     """View showing account details."""
 
-    # TODO: implement this view
+    message_cls = messages.Registration
+
+    @transaction.atomic
+    def acme_request(  # pylint: disable=unused-argument
+        self, message: messages.Registration, slug: Optional[str] = None
+    ) -> AcmeResponseAccount:
+        account = AcmeAccount.objects.get(slug=slug)
+
+        if message.status == AcmeAccount.STATUS_DEACTIVATED:
+            # RFC 8555, section 7.3.6 - Account Deactivation
+            log.info("Deactivating account %s", account.slug)
+            account.status = AcmeAccount.STATUS_DEACTIVATED
+            account.save()
+
+            # Cancel all pending operations
+            account.orders.filter(status=AcmeOrder.STATUS_PENDING).update(status=AcmeOrder.STATUS_INVALID)
+            AcmeAuthorization.objects.filter(
+                order__account=account, status=AcmeAuthorization.STATUS_PENDING
+            ).update(status=AcmeAuthorization.STATUS_DEACTIVATED)
+        elif message.contact:
+            self.validate_contacts(message)
+            account.contact = "\n".join(message.contact)
+            account.save()
+        else:
+            raise AcmeMalformed(message="Only contact information can be updated.")
+
+        return AcmeResponseAccount(self.request, account)
 
 
 class AcmeAccountOrdersView(AcmeBaseView):  # pylint: disable=abstract-method
     """View showing orders for an account (not yet implemented)"""
 
     # TODO: implement this view
+    def process_acme_request(self, slug: Optional[str]) -> AcmeResponse:  # pragma: no cover
+        raise AcmeException(message="Not Implemented.")
 
 
 class AcmeNewOrderView(AcmeMessageBaseView[NewOrder]):
@@ -611,9 +652,7 @@ class AcmeNewOrderView(AcmeMessageBaseView[NewOrder]):
         self, message: NewOrder, slug: Optional[str] = None
     ) -> AcmeResponseOrderCreated:
         """Process ACME request."""
-        now = timezone.now()
-        if timezone.is_naive(now):
-            now = timezone.make_aware(now, timezone=pytz.utc)
+        now = datetime.now(tz.utc)
 
         # josepy message classes define field names as class variables, but instance attributes are of the
         # same type (similar to Django). So we cast fields detected as RFC3339Field to datetime.
@@ -630,10 +669,11 @@ class AcmeNewOrderView(AcmeMessageBaseView[NewOrder]):
             # NOTE: Catches sending an empty tuple, which is not caught in message deserialization
             raise AcmeMalformed(message="The following fields are required: identifiers")
 
-        if not settings.USE_TZ and not_before:
-            not_before = timezone.make_naive(not_before)
-        if not settings.USE_TZ and not_after:
-            not_after = timezone.make_naive(not_after)
+        if settings.USE_TZ is False:
+            if not_before is not None:
+                not_before = make_naive(not_before)
+            if not_after is not None:
+                not_after = make_naive(not_after)
 
         # TODO: test if identifiers are acceptable
         order = AcmeOrder.objects.create(account=self.account, not_before=not_before, not_after=not_after)
@@ -644,8 +684,8 @@ class AcmeNewOrderView(AcmeMessageBaseView[NewOrder]):
         ]
 
         expires = order.expires
-        if timezone.is_naive(expires):  # acme.messages.Order requires a timezone-aware object
-            expires = timezone.make_aware(expires, timezone=pytz.utc)
+        if expires.tzinfo is None:  # acme.messages.Order requires a timezone-aware object
+            expires = expires.replace(tzinfo=tz.utc)
 
         response = AcmeResponseOrderCreated(
             authorizations=authorizations,
@@ -682,8 +722,8 @@ class AcmeOrderView(AcmePostAsGetView):
         # self.prepared['order'] = order.slug
 
         expires = order.expires
-        if timezone.is_naive(expires):  # acme.messages.Order requires a timezone-aware object
-            expires = timezone.make_aware(expires, timezone=pytz.utc)
+        if expires.tzinfo is None:  # acme.messages.Order requires a timezone-aware object
+            expires = expires.replace(tzinfo=tz.utc)
 
         authorizations = order.authorizations.all()
         if order.status in [AcmeOrder.STATUS_VALID, AcmeOrder.STATUS_INVALID]:
@@ -752,12 +792,15 @@ class AcmeOrderFinalizeView(AcmeMessageBaseView[CertificateRequest]):
             ).extension_type
         )
 
-        # Test if any subject Common Name is in the names for this order
-        # NOTE: certbot does *not* set name in the subject
-        csr_subject = Subject(csr.subject)
-        common_name = csr_subject.get("CN")
-        if isinstance(common_name, str) and x509.DNSName(common_name) not in names_from_order:
-            raise AcmeBadCSR(message="CommonName was not in order.")
+        # Perform sanity checks on the CSRs subject.
+        # NOTE: certbot does *not* set any subject at all
+        if csr.subject:
+            check_name(csr.subject)
+
+            # We allow a client setting a CommonName, but it *must* be part of the order.
+            common_name = next((attr for attr in csr.subject if attr.oid == NameOID.COMMON_NAME), None)
+            if common_name is not None and x509.DNSName(common_name.value) not in names_from_order:
+                raise AcmeBadCSR(message="CommonName was not in order.")
 
         try:
             names_from_csr: Set[x509.Name] = set(
@@ -799,8 +842,8 @@ class AcmeOrderFinalizeView(AcmeMessageBaseView[CertificateRequest]):
             raise AcmeForbidden(typ="orderNotReady", message="This order is not yet ready.")
 
         expires = order.expires
-        if timezone.is_naive(expires):  # acme.messages.Order requires a timezone-aware object
-            expires = timezone.make_aware(expires, timezone=pytz.utc)
+        if expires.tzinfo is None:  # acme.messages.Order requires a timezone-aware object
+            expires = expires.replace(tzinfo=tz.utc)
 
         authorizations = order.authorizations.all()
         for auth in authorizations:
@@ -852,8 +895,7 @@ class AcmeCertificateView(AcmePostAsGetView):
         # self.prepared['cert'] = slug
         # self.prepared['csr'] = cert.csr
         # self.prepared['order'] = cert.order.slug
-        bundle = "\n".join([c.pub.pem.strip() for c in cert.cert.bundle])
-        return HttpResponse(bundle, content_type="application/pem-certificate-chain")
+        return HttpResponse(cert.cert.bundle_as_pem, content_type="application/pem-certificate-chain")
 
 
 class AcmeAuthorizationView(AcmePostAsGetView):
@@ -890,8 +932,8 @@ class AcmeAuthorizationView(AcmePostAsGetView):
         challenges = auth.get_challenges()
 
         expires = auth.expires
-        if not settings.USE_TZ:  # acme.Order requires a timezone-aware object
-            expires = timezone.make_aware(expires, timezone=pytz.utc)
+        if expires.tzinfo is None:  # acme.Order requires a timezone-aware object
+            expires = expires.replace(tzinfo=tz.utc)
 
         # RFC8555, section 7.5.1:
         #
