@@ -49,7 +49,6 @@ from django.urls import path
 from django.urls import reverse
 from django.urls.resolvers import URLPattern
 from django.utils import timezone
-from django.utils.encoding import force_bytes
 from django.utils.html import escape
 from django.utils.html import format_html
 from django.utils.translation import gettext_lazy as _
@@ -84,6 +83,7 @@ from .models import X509CertMixin
 from .profiles import profiles
 from .querysets import CertificateQuerySet
 from .signals import post_issue_cert
+from .subject import Subject
 from .utils import OID_NAME_MAPPINGS
 from .utils import SERIAL_RE
 from .utils import add_colons
@@ -199,7 +199,7 @@ class CertificateMixin(
 
         if filetype == "PEM":
             if bundle is True:
-                data = "\n".join([cert.pub.pem.strip() for cert in obj.bundle]) + "\n"
+                data = obj.bundle_as_pem
             else:
                 data = obj.pub.pem
         elif filetype == "DER":
@@ -769,7 +769,7 @@ class CertificateAdmin(DjangoObjectActions, CertificateMixin[Certificate], Certi
         """Get initial data based on default profile.
 
         When resigning a certificate, get initial data from the certificate."""
-        data = super().get_changeform_initial_data(request)
+        data: typing.Dict[str, typing.Any] = super().get_changeform_initial_data(request)
 
         if hasattr(request, "_resign_obj"):
             # resign the cert, so we add initial data from the original cert
@@ -799,7 +799,9 @@ class CertificateAdmin(DjangoObjectActions, CertificateMixin[Certificate], Certi
                 "watchers": resign_obj.watchers.all(),
             }
         else:
-            data["subject"] = ca_settings.CA_PROFILES[ca_settings.CA_DEFAULT_PROFILE].get("subject", {})
+            data["subject"] = Subject(
+                ca_settings.CA_PROFILES[ca_settings.CA_DEFAULT_PROFILE].get("subject", {})
+            ).name
 
         return data
 
@@ -833,7 +835,7 @@ class CertificateAdmin(DjangoObjectActions, CertificateMixin[Certificate], Certi
             raise PermissionDenied
 
         try:
-            csr = x509.load_pem_x509_csr(force_bytes(request.POST["csr"]), default_backend())
+            csr = x509.load_pem_x509_csr(request.POST["csr"].encode("ascii"), default_backend())
         except Exception as e:  # pylint: disable=broad-except; docs don't list possible exceptions
             return JsonResponse({"message": str(e)}, status=HTTPStatus.BAD_REQUEST)
 
