@@ -19,6 +19,7 @@ import getpass
 import typing
 from datetime import timedelta
 
+from cryptography import x509
 from cryptography.hazmat.primitives import hashes
 from cryptography.hazmat.primitives.asymmetric import ec
 from cryptography.hazmat.primitives.serialization import Encoding
@@ -35,7 +36,8 @@ from ..utils import is_power2
 from ..utils import parse_encoding
 from ..utils import parse_hash_algorithm
 from ..utils import parse_key_curve
-from ..utils import shlex_split
+from ..utils import split_str
+from ..utils import x509_name
 
 ActionType = typing.TypeVar("ActionType")
 
@@ -325,6 +327,24 @@ class SubjectAction(SingleValueAction[Subject]):
             raise argparse.ArgumentError(self, str(e))
 
 
+class NameAction(SingleValueAction[x509.Name]):
+    """Action to parse a string into a :py:class:`cg:~cryptography.x509.Name`.
+
+    Note that this action does *not* take care of sorting the subject in any way.
+
+    >>> parser.add_argument('--name', action=NameAction)  # doctest: +ELLIPSIS
+    NameAction(...)
+    >>> parser.parse_args(["--name", "/CN=example.com"])
+    Namespace(name=<Name(CN=example.com)>)
+    """
+
+    def parse_value(self, value: str) -> x509.Name:
+        try:
+            return x509_name(value)
+        except ValueError as e:
+            raise argparse.ArgumentError(self, str(e))
+
+
 class URLAction(SingleValueAction[str]):
     """Action to pass a single valid URL.
 
@@ -396,7 +416,7 @@ class OrderedSetExtensionAction(ExtensionAction):
     ) -> None:
         ext = self.extension()
 
-        ext_values = shlex_split(values, ", ")
+        ext_values = list(split_str(values, ", "))
         if ext_values[0] == "critical":
             ext_values = ext_values[1:]
             ext.critical = True
