@@ -16,6 +16,9 @@
 .. seealso:: https://docs.djangoproject.com/en/dev/howto/custom-management-commands/
 """
 
+import argparse
+import typing
+
 from ...tasks import cache_crls
 from ...tasks import run_task
 from ..base import BaseCommand
@@ -24,9 +27,13 @@ from ..base import BaseCommand
 class Command(BaseCommand):  # pylint: disable=missing-class-docstring
     help = "Cache CRLs"
 
-    def add_arguments(self, parser):
-        parser.add_argument('serial', nargs='*',
-                            help="Generate CRLs for the given CAs. If omitted, generate CRLs for all CAs.")
+    def add_arguments(self, parser: argparse.ArgumentParser) -> None:
+        parser.add_argument(
+            "serial",
+            nargs="*",
+            help="Generate CRLs for the given CAs. If omitted, generate CRLs for all CAs.",
+        )
 
-    def handle(self, **options):  # pylint: disable=arguments-differ
-        run_task(cache_crls, options['serial'])
+    def handle(self, *args: typing.Any, **options: typing.Any) -> None:
+        serials = typing.cast(typing.List[str], options["serial"])
+        run_task(cache_crls, serials)
