@@ -18,19 +18,16 @@
 
 import sys
 import typing
-import warnings
 from datetime import timedelta
 
 from cryptography import x509
 from cryptography.hazmat.backends import default_backend
-from cryptography.hazmat.primitives.serialization import Encoding
 
 from django.core.management.base import CommandError
 from django.core.management.base import CommandParser
 from django.utils import timezone
 
 from ... import ca_settings
-from ...deprecation import RemovedInDjangoCA120Warning
 from ...extensions import Extension
 from ...extensions import SubjectAlternativeName
 from ...management.base import BaseSignCommand
@@ -95,11 +92,8 @@ https://django-ca.readthedocs.io/en/latest/extensions.html for more information.
             metavar="FILE",
             help="The path to the certificate to sign, if ommitted, you will be be prompted.",
         )
-        self.add_format(
-            parser,
-            opts=["--csr-format"],
-            default=None,
-            help_text="Format of the CSR (DEPRECATED, format is now auto-detected).",
+        parser.add_argument(
+            "-b", "--bundle", default=False, action="store_true", help="Output the whole certificate bundle."
         )
 
         self.add_profile(
@@ -115,9 +109,9 @@ https://django-ca.readthedocs.io/en/latest/extensions.html for more information.
         expires: timedelta,
         watch: typing.List[str],
         password: typing.Optional[bytes],
-        encoding: typing.Optional[Encoding],
         cn_in_san: bool,
         csr_path: str,
+        bundle: bool,
         profile: typing.Optional[str],
         out: typing.Optional[str],
         **options: typing.Any,
@@ -126,11 +120,6 @@ https://django-ca.readthedocs.io/en/latest/extensions.html for more information.
             raise CommandError("Certificate Authority has expired.")
         if ca.revoked:
             raise CommandError("Certificate Authority is revoked.")
-        if encoding is not None:
-            warnings.warn(
-                "--csr-format option is deprecated and will be removed in django-ca 1.20.0.",
-                category=RemovedInDjangoCA120Warning,
-            )
         self.test_options(ca=ca, expires=expires, password=password, **options)
         subject = subject or Subject()
 
@@ -182,8 +171,13 @@ https://django-ca.readthedocs.io/en/latest/extensions.html for more information.
 
         cert.watchers.add(*watchers)
 
+        if bundle is True:
+            output = cert.bundle_as_pem
+        else:
+            output = cert.pub.pem
+
         if out:
             with open(out, "w", encoding="ascii") as stream:
-                stream.write(cert.pub.pem)
+                stream.write(output)
         else:
-            self.stdout.write(cert.pub.pem)
+            self.stdout.write(output)
