@@ -27,7 +27,6 @@ from typing import Sequence
 from typing import Tuple
 from typing import TypeVar
 from typing import Union
-from typing import cast
 
 from cryptography import x509
 from cryptography.hazmat.backends import default_backend
@@ -40,10 +39,9 @@ from cryptography.x509.oid import AuthorityInformationAccessOID
 from django.core.files.base import ContentFile
 from django.db import models
 from django.urls import reverse
-from django.utils.encoding import force_str
 
 from . import ca_settings
-from .deprecation import RemovedInDjangoCA120Warning
+from .deprecation import RemovedInDjangoCA122Warning
 from .extensions import Extension
 from .extensions import IssuerAlternativeName
 from .extensions import NameConstraints
@@ -57,20 +55,19 @@ from .signals import post_issue_cert
 from .signals import pre_create_ca
 from .subject import Subject
 from .typehints import Expires
-from .typehints import Literal
 from .typehints import ParsableExtension
 from .typehints import ParsableHash
-from .typehints import ParsableSubject
-from .typehints import PrivateKeyTypes
+from .typehints import ParsableKeyType
 from .utils import ca_storage
 from .utils import generate_private_key
 from .utils import get_cert_builder
 from .utils import int_to_hex
-from .utils import parse_csr
 from .utils import parse_expires
 from .utils import parse_hash_algorithm
+from .utils import sort_name
 from .utils import validate_hostname
 from .utils import validate_key_parameters
+from .utils import x509_name
 
 # https://mypy.readthedocs.io/en/latest/runtime_troubles.html
 if TYPE_CHECKING:
@@ -149,7 +146,7 @@ class CertificateManagerMixin(Generic[X509CertMixinTypeVar, QuerySetTypeVar]):
 
         extensions: List[Tuple[bool, Union[x509.CRLDistributionPoints, x509.AuthorityInformationAccess]]] = []
         if crl_url:
-            urls = [x509.UniformResourceIdentifier(force_str(c)) for c in crl_url]
+            urls = [x509.UniformResourceIdentifier(c) for c in crl_url]
             dps = [
                 x509.DistributionPoint(full_name=[c], relative_name=None, crl_issuer=None, reasons=None)
                 for c in urls
@@ -157,12 +154,12 @@ class CertificateManagerMixin(Generic[X509CertMixinTypeVar, QuerySetTypeVar]):
             extensions.append((False, x509.CRLDistributionPoints(dps)))
         auth_info_access = []
         if ocsp_url:
-            uri = x509.UniformResourceIdentifier(force_str(ocsp_url))
+            uri = x509.UniformResourceIdentifier(ocsp_url)
             auth_info_access.append(
                 x509.AccessDescription(access_method=AuthorityInformationAccessOID.OCSP, access_location=uri)
             )
         if issuer_url:
-            uri = x509.UniformResourceIdentifier(force_str(issuer_url))
+            uri = x509.UniformResourceIdentifier(issuer_url)
             auth_info_access.append(
                 x509.AccessDescription(
                     access_method=AuthorityInformationAccessOID.CA_ISSUERS, access_location=uri
@@ -219,7 +216,7 @@ class CertificateAuthorityManager(
     def init(
         self,
         name: str,
-        subject: Union[ParsableSubject, Subject],
+        subject: x509.Name,
         expires: Expires = None,
         algorithm: ParsableHash = None,
         parent: Optional["CertificateAuthority"] = None,
@@ -236,7 +233,7 @@ class CertificateAuthorityManager(
         password: Optional[bytes] = None,
         parent_password: Optional[Union[str, bytes]] = None,
         ecc_curve: Optional[ec.EllipticCurve] = None,
-        key_type: Literal["RSA", "DSA", "ECC", "EdDSA"] = "RSA",
+        key_type: ParsableKeyType = "RSA",
         key_size: Optional[int] = None,
         extra_extensions: Optional[
             typing.Collection[Union["x509.Extension[x509.ExtensionType]", "Extension[Any, Any, Any]"]]
@@ -251,15 +248,21 @@ class CertificateAuthorityManager(
     ) -> "CertificateAuthority":
         """Create a new certificate authority.
 
+        .. deprecated:: 1.20.0
+
+           Passing unparsed values is deprecated and will be removed in ``django_ca==1.22``. This affects the
+           following parameters:
+
+           * Passing a ``str`` or :py:class:`~django_ca.subject.Subject` for ``subject``.
+
+
         Parameters
         ----------
 
         name : str
             The name of the CA. This is a human-readable string and is used for administrative purposes only.
-        subject : dict or str or :py:class:`~django_ca.subject.Subject`
-            Subject string, e.g. ``"/CN=example.com"`` or ``Subject("/CN=example.com")``. The value is
-            actually passed to :py:class:`~django_ca.subject.Subject` if it is not already an instance of that
-            class.
+        subject : :py:class:`cg:cryptography.x509.Name`
+           The desired subject for the certificate.
         expires : int or datetime or timedelta, optional
             When this certificate authority will expire, defaults to :ref:`CA_DEFAULT_EXPIRES
             <settings-ca-default-expires>`.
@@ -303,9 +306,9 @@ class CertificateAuthorityManager(
             Password to encrypt the private key with.
         parent_password : bytes, optional
             Password that the private key of the parent CA is encrypted with.
-        ecc_curve : str or EllipticCurve, optional
-            The elliptic curve to use for ECC type keys, passed verbatim to
-            :py:func:`~django_ca.utils.parse_key_curve`.
+        ecc_curve : :py:class:`~cg:cryptography.hazmat.primitives.asymmetric.ec.EllipticCurve`, optional
+            An elliptic curve to use for ECC keys. This parameter is ignored if ``key_type`` is not ``"ECC"``.
+            Defaults to the :ref:`CA_DEFAULT_ECC_CURVE <settings-ca-default-ecc-curve>`.
         key_type: str, optional
             The type of private key to generate, must be one of ``"RSA"``, ``"DSA"``, ``"ECC"``, or
             ``"EdDSA"`` , with ``"RSA"`` being the default.
@@ -341,8 +344,9 @@ class CertificateAuthorityManager(
         """
         # pylint: disable=too-many-arguments,too-many-locals,too-many-branches,too-many-statements
         # NOTE: Already verified by KeySizeAction, so these checks are only for when the Python API is used
-        #       directly.
-        generate_key_args = validate_key_parameters(key_size, key_type, ecc_curve)
+        #       directly. generate_private_key() invokes this again, but we here to avoid sending a signal.
+        validate_key_parameters(key_size, key_type, ecc_curve)
+
         if not openssh_ca:
             algorithm = parse_hash_algorithm(algorithm)
         else:
@@ -365,9 +369,24 @@ class CertificateAuthorityManager(
             # TYPE NOTE: This seems to be a false positive
             extra_extensions.extend([SshHostCaExtension(), SshUserCaExtension()])  # type: ignore[list-item]
 
+        # Handle old types for subject
+        _subject_warning = f"Passing a {subject.__class__.__name__} as subject is deprecated"
+        if isinstance(subject, Subject):
+            warnings.warn(
+                _subject_warning,
+                category=RemovedInDjangoCA122Warning,
+                stacklevel=1,
+            )
+            subject = subject.name
+        elif not isinstance(subject, x509.Name):
+            warnings.warn(
+                _subject_warning,
+                category=RemovedInDjangoCA122Warning,
+                stacklevel=1,
+            )
+            subject = sort_name(x509_name(subject))
+
         # Normalize extensions to django_ca.extensions.Extension subclasses
-        if not isinstance(subject, Subject):
-            subject = Subject(subject)
         if issuer_alt_name and not isinstance(issuer_alt_name, IssuerAlternativeName):
             issuer_alt_name = IssuerAlternativeName({"value": [issuer_alt_name]})
         if crl_url is None:
@@ -438,16 +457,11 @@ class CertificateAuthorityManager(
             acme_requires_contact=acme_requires_contact,
         )
 
-        # NOTE: Since priv_key_params is a union of tuples, mypy is unable to determine that all overloaded
-        # variants for generate_private_key() potentially match, and thus matches the first variant. See:
-        #   https://mypy.readthedocs.io/en/latest/more_types.html#type-checking-calls-to-overloads
-        private_key = cast(PrivateKeyTypes, generate_private_key(*generate_key_args))
+        private_key = generate_private_key(key_size, key_type, ecc_curve)
         public_key = private_key.public_key()
 
         builder = get_cert_builder(expires, serial=serial)
         builder = builder.public_key(public_key)
-        subject = subject.name
-
         builder = builder.subject_name(subject)
         builder = builder.add_extension(x509.BasicConstraints(ca=True, path_length=pathlen), critical=True)
         builder = builder.add_extension(
@@ -554,23 +568,12 @@ class CertificateManager(
         self,
         ca: "CertificateAuthority",
         csr: x509.CertificateSigningRequest,
-        csr_format: Encoding = Encoding.PEM,
         profile: Optional[Profile] = None,
         autogenerated: Optional[bool] = None,
         **kwargs: Any,
     ) -> "Certificate":
         """Create and sign a new certificate based on the given profile.
 
-        .. deprecated:: 1.18.0
-
-           Since ``django-ca==1.18.0``, passing unparsed values is deprecated and will be removed in
-           ``django-ca==1.20.0``. This affects the following parameters:
-
-           * The `csr_format` parameter will be removed (because CSRs must already be parsed).
-           * Passing ``str`` or ``bytes`` for the `csr` parameter (pass a
-             :py:class:`~cg:cryptography.x509.CertificateSigningRequest` instead).
-           * ``str`` for the `profile` parameter (pass a :py:class:`~django_ca.profiles.Profile` instead).
-
         Parameters
         ----------
 
@@ -579,9 +582,6 @@ class CertificateManager(
         csr : :py:class:`~cg:cryptography.x509.CertificateSigningRequest`
             The certificate signing request to use when signing a certificate. Passing a ``str`` or ``bytes``
             is deprecated and will be removed in django-ca 1.20.0.
-        csr_format : :py:class:`~cg:cryptography.hazmat.primitives.serialization.Encoding`, optional
-            If passing `csr` as ``str`` or ``bytes``, the format of the certificate signing request. This
-            parameter will be removed in django-ca 1.20.0.
         profile : :py:class:`~django_ca.profiles.Profile`, optional
             The name of a profile or a manually created :py:class:`~django_ca.profiles.Profile` instance. If
             not given, the profile configured by :ref:`CA_DEFAULT_PROFILE <settings-ca-default-profile>` is
@@ -596,22 +596,9 @@ class CertificateManager(
         # Get the profile object if none was passed
         if profile is None:
             profile = profiles[None]
-        elif isinstance(profile, str):
-            warnings.warn(
-                "Passing a str as a profile is deprecated and will be removed in django-ca==1.20.0.",
-                category=RemovedInDjangoCA120Warning,
-                stacklevel=2,
-            )
-            profile = profiles[profile]
         elif not isinstance(profile, Profile):
             raise TypeError("profile must be of type django_ca.profiles.Profile.")
 
-        if not isinstance(csr, x509.CertificateSigningRequest):
-            clsname = type(csr).__name__
-            msg = f"Passing {clsname} as csr is deprecated, pass an x509.CertificateSigningRequest instead."
-            warnings.warn(msg, category=RemovedInDjangoCA120Warning, stacklevel=2)
-            csr = parse_csr(csr, csr_format=csr_format)
-
         cert = profile.create_cert(ca, csr, **kwargs)
 
         obj = self.model(ca=ca, csr=LazyCertificateSigningRequest(csr), profile=profile.name)
