@@ -27,6 +27,7 @@ import typing
 import warnings
 from datetime import datetime
 from datetime import timedelta
+from datetime import timezone as tz
 from typing import Dict
 from typing import Iterable
 from typing import List
@@ -35,18 +36,20 @@ from typing import Tuple
 from typing import Union
 from typing import cast
 
-import pytz
-
 from cryptography import x509
 from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives import hashes
 from cryptography.hazmat.primitives import serialization
 from cryptography.hazmat.primitives.asymmetric import dsa
+from cryptography.hazmat.primitives.asymmetric import ec
 from cryptography.hazmat.primitives.asymmetric import ed448
+from cryptography.hazmat.primitives.asymmetric import x448
+from cryptography.hazmat.primitives.asymmetric import x25519
 from cryptography.hazmat.primitives.serialization import Encoding
 from cryptography.hazmat.primitives.serialization import PrivateFormat
 from cryptography.hazmat.primitives.serialization import PublicFormat
 from cryptography.hazmat.primitives.serialization import load_pem_private_key
+from cryptography.x509.oid import NameOID
 
 from django.conf import settings
 from django.core.cache import cache
@@ -67,7 +70,7 @@ from .acme.constants import BASE64_URL_ALPHABET
 from .acme.constants import IdentifierType
 from .acme.constants import Status
 from .constants import ReasonFlags
-from .deprecation import RemovedInDjangoCA120Warning
+from .deprecation import RemovedInDjangoCA122Warning
 from .extensions import OID_TO_EXTENSION
 from .extensions import AuthorityInformationAccess
 from .extensions import AuthorityKeyIdentifier
@@ -111,16 +114,13 @@ from .querysets import CertificateAuthorityQuerySet
 from .querysets import CertificateQuerySet
 from .signals import post_revoke_cert
 from .signals import pre_revoke_cert
-from .subject import Subject
+from .typehints import PRIVATE_KEY_TYPES
 from .typehints import Expires
 from .typehints import ExtensionTypeTypeVar
 from .typehints import Literal
-from .typehints import ParsableGeneralNameList
 from .typehints import ParsableHash
-from .typehints import ParsableKeyCurve
 from .typehints import ParsableKeyType
 from .typehints import ParsableValue
-from .typehints import PrivateKeyTypes
 from .typehints import SerializedValue
 from .utils import bytes_to_hex
 from .utils import ca_storage
@@ -132,8 +132,8 @@ from .utils import int_to_hex
 from .utils import multiline_url_validator
 from .utils import parse_encoding
 from .utils import parse_expires
-from .utils import parse_general_name
 from .utils import parse_hash_algorithm
+from .utils import parse_key_curve
 from .utils import read_file
 from .utils import validate_key_parameters
 
@@ -299,6 +299,13 @@ class X509CertMixin(DjangoCAModel):
     class Meta:
         abstract = True
 
+    @property
+    def bundle_as_pem(self) -> str:
+        """Get the bundle as PEM."""
+        # TYPE NOTE: bundle is defined in base class but returns a list (considered invariant by mypy). This
+        #            means that an abstract "bundle" property here could not be correctly typed.
+        return "".join(c.pub.pem for c in self.bundle)  # type:  ignore[attr-defined]
+
     def get_revocation_reason(self) -> Optional[x509.ReasonFlags]:
         """Get the revocation reason of this certificate."""
         if self.revoked is False:
@@ -316,7 +323,7 @@ class X509CertMixin(DjangoCAModel):
 
         if timezone.is_aware(self.compromised):
             # convert datetime object to UTC and make it naive
-            return timezone.make_naive(self.compromised, pytz.utc)
+            return timezone.make_naive(self.compromised, tz.utc)
 
         return self.compromised
 
@@ -330,19 +337,24 @@ class X509CertMixin(DjangoCAModel):
 
         if timezone.is_aware(self.revoked_date):
             # convert datetime object to UTC and make it naive
-            return timezone.make_naive(self.revoked_date, pytz.utc)
+            return timezone.make_naive(self.revoked_date, tz.utc)
 
         return self.revoked_date
 
     def update_certificate(self, value: x509.Certificate) -> None:
-        """Update this instance with data from a :py:class:`cg:cryptography.x509.Certificate`."""
+        """Update this instance with data from a :py:class:`cg:cryptography.x509.Certificate`.
+
+        This function will also populate the `cn`, `serial, `expires` and `valid_from` fields.
+        """
         self.pub = LazyCertificate(value)
-        self.cn = cast(str, self.subject.get("CN", ""))  # pylint: disable=invalid-name
+        self.cn = next(  # pylint: disable=invalid-name
+            (attr.value for attr in value.subject if attr.oid == NameOID.COMMON_NAME), ""
+        )
         self.expires = self.not_after
         self.valid_from = self.not_before
         if settings.USE_TZ:
-            self.expires = timezone.make_aware(self.expires, timezone=pytz.utc)
-            self.valid_from = timezone.make_aware(self.valid_from, timezone=pytz.utc)
+            self.expires = timezone.make_aware(self.expires, timezone=tz.utc)
+            self.valid_from = timezone.make_aware(self.valid_from, timezone=tz.utc)
 
         self.serial = int_to_hex(value.serial_number)
 
@@ -436,9 +448,14 @@ class X509CertMixin(DjangoCAModel):
         return base64.b64encode(public_key_hash).decode("utf-8")
 
     @property
-    def issuer(self) -> Subject:
-        """The certificate issuer field as :py:class:`~django_ca.subject.Subject`."""
-        return Subject([(s.oid, s.value) for s in self.pub.loaded.issuer])
+    def issuer(self) -> x509.Name:
+        """The certificate issuer field as :py:class:`~cg:cryptography.x509.Name`.
+
+        .. versionchanged:: 1.20.0
+
+           This property was a :py:class:`~django_ca.subject.Subject` before ``django-ca==1.20.0``.
+        """
+        return self.pub.loaded.issuer
 
     @property
     def not_before(self) -> datetime:
@@ -476,9 +493,14 @@ class X509CertMixin(DjangoCAModel):
         post_revoke_cert.send(sender=self.__class__, cert=self)
 
     @property
-    def subject(self) -> Subject:
-        """The certificates subject as :py:class:`~django_ca.subject.Subject`."""
-        return Subject([(s.oid, s.value) for s in self.pub.loaded.subject])
+    def subject(self) -> x509.Name:
+        """The certificate subject field as :py:class:`~cg:cryptography.x509.Name`.
+
+        .. versionchanged:: 1.20.0
+
+           This property was a :py:class:`~django_ca.subject.Subject` before ``django-ca==1.20.0``.
+        """
+        return self.pub.loaded.subject
 
     @property
     def distinguished_name(self) -> str:
@@ -788,7 +810,7 @@ class CertificateAuthority(X509CertMixin):
 
     _key = None
 
-    def key(self, password: Optional[Union[str, bytes]]) -> PrivateKeyTypes:
+    def key(self, password: Optional[Union[str, bytes]] = None) -> PRIVATE_KEY_TYPES:
         """The CAs private key as private key.
 
         .. seealso:: :py:func:`~cg:cryptography.hazmat.primitives.serialization.load_pem_private_key`.
@@ -799,7 +821,11 @@ class CertificateAuthority(X509CertMixin):
         if self._key is None:
             key_data = read_file(self.private_key_path)
 
-            self._key = load_pem_private_key(key_data, password, default_backend())
+            try:
+                self._key = load_pem_private_key(key_data, password, default_backend())
+            except ValueError as ex:
+                # cryptography passes the OpenSSL error directly here and it is notoriously unstable.
+                raise ValueError("Could not decrypt private key - bad password?") from ex
         if isinstance(self._key, ed448.Ed448PrivateKey):  # pragma: nocover
             raise ValueError("Ed448 private keys are not supported.")
 
@@ -876,12 +902,19 @@ class CertificateAuthority(X509CertMixin):
         algorithm: ParsableHash = None,
         password: Optional[Union[str, bytes]] = None,
         key_size: Optional[int] = None,
-        key_type: Optional[ParsableKeyType] = None,
-        ecc_curve: ParsableKeyCurve = None,
+        key_type: ParsableKeyType = "RSA",
+        ecc_curve: Optional[ec.EllipticCurve] = None,
         autogenerated: bool = True,
     ) -> Tuple[str, str, "Certificate"]:
         """Generate OCSP keys for this CA.
 
+        .. deprecated:: 1.20.0
+
+           Passing unparsed values is deprecated and will be removed in ``django_ca==1.22``. This affects the
+           following parameters:
+
+           * Passing a ``str`` for ``ecc_curve``.
+
         Parameters
         ----------
 
@@ -898,11 +931,11 @@ class CertificateAuthority(X509CertMixin):
         key_size : int, optional
             The key size of the private key, defaults to :ref:`CA_DEFAULT_KEY_SIZE
             <settings-ca-default-key-size>`.
-        key_type : {"RSA", "DSA", "ECC", "EdDSA"}, optional
+        key_type : {"RSA", "DSA", "ECC", "EdDSA", "Ed448"}, optional
             The private key type to use, the default is ``"RSA"``.
-        ecc_curve : str, optional
-            Passed to :py:func:`~django_ca.utils.parse_key_curve`, defaults to the :ref:`CA_DEFAULT_ECC_CURVE
-            <settings-ca-default-ecc-curve>`.
+        ecc_curve : :py:class:`~cg:cryptography.hazmat.primitives.asymmetric.ec.EllipticCurve`, optional
+            An elliptic curve to use for ECC keys. This parameter is ignored if ``key_type`` is not ``"ECC"``.
+            Defaults to the :ref:`CA_DEFAULT_ECC_CURVE <settings-ca-default-ecc-curve>`.
         autogenerated : bool, optional
             Set the ``autogenerated`` flag of the certificate. ``True`` by default, since this method is
             usually invoked by regular cron jobs.
@@ -911,22 +944,24 @@ class CertificateAuthority(X509CertMixin):
         # OCSP is pretty complex, there is no way to trim down the arguments w/o losing features.
 
         password = password or self.get_password()
-        if key_type is None:
-            ca_key = self.key(password)
-            if isinstance(ca_key, dsa.DSAPrivateKey):
-                key_type = "DSA"
-                algorithm = "SHA1"
 
-        priv_key_params = validate_key_parameters(key_size, key_type, ecc_curve)
+        # DSA private keys can only sign DSA keys, hash algorithm for DSA keys must be SHA1
+        if isinstance(self.key(password), dsa.DSAPrivateKey):
+            key_type = "DSA"
+        if key_type == "DSA":
+            algorithm = hashes.SHA1()
+
+        if key_type == "ECC" and isinstance(ecc_curve, str):
+            warnings.warn("Passing a str as ecc_curve is deprecated", category=RemovedInDjangoCA122Warning)
+            ecc_curve = parse_key_curve(ecc_curve)
+
+        validate_key_parameters(key_size, key_type, ecc_curve)
         expires = parse_expires(expires)
         algorithm = parse_hash_algorithm(algorithm)
         safe_serial = self.serial.replace(":", "")
 
         # generate the private key
-        # NOTE: Since priv_key_params is a union of tuples, mypy is unable to determine that all overloaded
-        # variants for generate_private_key() potentially match, and thus matches the first variant. See:
-        #   https://mypy.readthedocs.io/en/latest/more_types.html#type-checking-calls-to-overloads
-        private_key = cast(PrivateKeyTypes, generate_private_key(*priv_key_params))
+        private_key = generate_private_key(key_size, key_type, ecc_curve)
         private_pem = private_key.private_bytes(
             encoding=Encoding.PEM,
             format=PrivateFormat.PKCS8,
@@ -975,8 +1010,12 @@ class CertificateAuthority(X509CertMixin):
 
         try:
             ski = self.pub.loaded.extensions.get_extension_for_class(x509.SubjectKeyIdentifier)
-        except x509.ExtensionNotFound:
-            return x509.AuthorityKeyIdentifier.from_issuer_public_key(self.pub.loaded.public_key())
+        except x509.ExtensionNotFound as ex:
+            public_key = self.pub.loaded.public_key()
+            if isinstance(public_key, (x448.X448PublicKey, x25519.X25519PublicKey)):  # pragma: no cover
+                # COVERAGE NOTE: This does not happen in reality, we never generate keys of this type
+                raise TypeError("Cannot get AuthorityKeyIdentifier for X25519 or X448 keys.") from ex
+            return x509.AuthorityKeyIdentifier.from_issuer_public_key(public_key)
         else:
             return x509.AuthorityKeyIdentifier.from_issuer_subject_key_identifier(ski.value)
 
@@ -1022,7 +1061,7 @@ class CertificateAuthority(X509CertMixin):
         password: Optional[Union[str, bytes]] = None,
         scope: Optional[Literal["ca", "user", "attribute"]] = None,
         counter: Optional[str] = None,
-        full_name: Optional[ParsableGeneralNameList] = None,
+        full_name: Optional[typing.Iterable[x509.GeneralName]] = None,
         relative_name: Optional[x509.RelativeDistinguishedName] = None,
     ) -> x509.CertificateRevocationList:
         """Generate a Certificate Revocation List (CRL).
@@ -1032,13 +1071,10 @@ class CertificateAuthority(X509CertMixin):
         The former defaults to the ``crl_url`` field, pass ``None`` to not include the value. At most one of
         the two may be set.
 
-        .. deprecated:: 1.18.0
+        .. versionchanged:: 1.20.0
 
-           Since ``django-ca==1.18.0``, passing unparsed values is deprecated and will be removed in
-           ``django-ca==1.20.0``. This affects the following parameters:
-
-           * Passing a ``str`` as `algorithm` (pass a
-             :py:class:`~cg:cryptography.hazmat.primitives.hashes.HashAlgorithm` instead).
+           The ``full_name`` parameter must be a list of :py:class:`~cg:cryptography.x509.GeneralName`,
+           ``str`` are no longer allowed.
 
         Parameters
         ----------
@@ -1058,7 +1094,7 @@ class CertificateAuthority(X509CertMixin):
             Override the counter-variable for the CRL Number extension. Passing the same key to multiple
             invocations will yield a different sequence then what would ordinarily be returned. The default is
             to use the scope as the key.
-        full_name : list of str or :py:class:`~cg:cryptography.x509.GeneralName`, optional
+        full_name : list of :py:class:`~cg:cryptography.x509.GeneralName`, optional
             List of general names to use in the Issuing Distribution Point extension. If not passed, use
             ``crl_url`` if set.
         relative_name : :py:class:`~cg:cryptography.x509.RelativeDistinguishedName`, optional
@@ -1076,16 +1112,9 @@ class CertificateAuthority(X509CertMixin):
         now = now_builder = timezone.now()
         if algorithm is None:
             algorithm = ca_settings.CA_DIGEST_ALGORITHM
-        elif isinstance(algorithm, str):
-            warnings.warn(
-                "Passing a str as algorithm is deprecated and will be removed in django-ca==1.20.0.",
-                category=RemovedInDjangoCA120Warning,
-                stacklevel=2,
-            )
-            algorithm = parse_hash_algorithm(algorithm)
 
         if timezone.is_aware(now_builder):
-            now_builder = timezone.make_naive(now, pytz.utc)
+            now_builder = timezone.make_naive(now, tz.utc)
         else:
             now_builder = datetime.utcnow()
 
@@ -1096,12 +1125,12 @@ class CertificateAuthority(X509CertMixin):
 
         parsed_full_name = None
         if full_name is not None:
-            parsed_full_name = [parse_general_name(n) for n in full_name]
+            parsed_full_name = full_name
 
-        # CRLs for root CAs with scope "ca" (or no scope) do not add an IssuingDistributionPoint extension by
-        # default. For full path validation with CRLs, the CRL is also used for validating the Root CA (which
-        # does not contain a CRL Distribution Point). But the Full Name in the CRL IDP and the CA CRL DP have
-        # to match. See also:
+        # CRLs for root CAs with scope "ca" (or no scope - this includes CAs) do not sete a full_name in the
+        # IssuingDistributionPoint extension by default. For full path validation with CRLs, the CRL is also
+        # used for validating the Root CA (which does not contain a CRL Distribution Point). But the Full Name
+        # in the CRL IDP and the CA CRL DP have to match. See also:
         #       https://github.com/mathiasertl/django-ca/issues/64
         elif scope in ("ca", None) and self.parent is None:
             parsed_full_name = None
@@ -1694,7 +1723,7 @@ class AcmeChallenge(DjangoCAModel):
             return None
 
         if timezone.is_naive(self.validated):
-            return timezone.make_aware(self.validated, timezone=pytz.UTC)
+            return timezone.make_aware(self.validated, timezone=tz.utc)
         return self.validated
 
     @property
