@@ -31,7 +31,6 @@ from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives.hashes import HashAlgorithm
 
 from . import ca_settings
-from .deprecation import RemovedInDjangoCA120Warning
 from .deprecation import RemovedInDjangoCA121Warning
 from .extensions import KEY_TO_EXTENSION
 from .extensions import AuthorityInformationAccess
@@ -52,7 +51,7 @@ from .utils import get_cert_builder
 from .utils import parse_expires
 from .utils import parse_general_name
 from .utils import parse_hash_algorithm
-from .utils import shlex_split
+from .utils import split_str
 
 if TYPE_CHECKING:
     from .models import CertificateAuthority
@@ -128,23 +127,23 @@ class Profile:
         # set some sane extension defaults
         self.extensions.setdefault(BasicConstraints.key, BasicConstraints())
 
-    def __eq__(self, o: object) -> bool:
-        if not isinstance(o, (Profile, DefaultProfileProxy)):
+    def __eq__(self, value: object) -> bool:
+        if not isinstance(value, (Profile, DefaultProfileProxy)):
             return False
-        algo = isinstance(o.algorithm, type(self.algorithm))
+        algo = isinstance(value.algorithm, type(self.algorithm))
 
         return (
-            self.name == o.name
-            and self.subject == o.subject
+            self.name == value.name
+            and self.subject == value.subject
             and algo
-            and self.extensions == o.extensions
-            and self.cn_in_san == o.cn_in_san
-            and self.expires == o.expires
-            and self.add_crl_url == o.add_crl_url
-            and self.add_issuer_url == o.add_issuer_url
-            and self.add_ocsp_url == o.add_ocsp_url
-            and self.add_issuer_alternative_name == o.add_issuer_alternative_name
-            and self.description == o.description
+            and self.extensions == value.extensions
+            and self.cn_in_san == value.cn_in_san
+            and self.expires == value.expires
+            and self.add_crl_url == value.add_crl_url
+            and self.add_issuer_url == value.add_issuer_url
+            and self.add_ocsp_url == value.add_ocsp_url
+            and self.add_issuer_alternative_name == value.add_issuer_alternative_name
+            and self.description == value.description
         )
 
     def _parse_extension_value(
@@ -185,13 +184,6 @@ class Profile:
     ) -> x509.Certificate:
         """Create a x509 certificate based on this profile, the passed CA and input parameters.
 
-        .. deprecated:: 1.18.0
-
-           Since ``django-ca==1.18.0``, passing unparsed values is deprecated and will be removed in
-           ``django-ca==1.20.0``. This affects the following parameters:
-
-           * Passing a ``str`` as `algorithm`.
-
         This function is the core function used to create x509 certificates. In it's simplest form, you only
         need to pass a ca, a CSR and a subject to get a valid certificate::
 
@@ -303,13 +295,6 @@ class Profile:
             expires = self.expires
         if algorithm is None:
             algorithm = self.algorithm
-        elif isinstance(algorithm, str):
-            warnings.warn(
-                "Passing a str as algorithm is deprecated and will be removed in django-ca==1.20.0.",
-                category=RemovedInDjangoCA120Warning,
-                stacklevel=2,
-            )
-            algorithm = parse_hash_algorithm(algorithm)
 
         # Make sure that expires is a fixed timestamp
         expires = parse_expires(expires)
@@ -337,7 +322,7 @@ class Profile:
         public_key = csr.public_key()
         builder = get_cert_builder(expires)
         builder = builder.public_key(public_key)
-        builder = builder.issuer_name(ca.pub.loaded.subject)
+        builder = builder.issuer_name(ca.subject)
         builder = builder.subject_name(cert_subject.name)
 
         for _key, extension in cert_extensions.items():
@@ -420,7 +405,7 @@ class Profile:
                     f"extensions[{IssuerAlternativeName.key}] is not of type IssuerAlternativeName"
                 )
 
-            ian.extend(shlex_split(ca.issuer_alt_name, ","))
+            ian.extend(split_str(ca.issuer_alt_name, ","))
             extensions[IssuerAlternativeName.key] = ian
 
     def _update_san_from_cn(
