@@ -1,4 +1,4 @@
-// see https://docs.djangoproject.com/en/1.9/ref/csrf/#ajax
+// see https://docs.djangoproject.com/en/dev/ref/csrf/#ajax
 function getCookie(name) {
     var cookieValue = null;
     if (document.cookie && document.cookie != '') {
@@ -22,8 +22,9 @@ function csrfSafeMethod(method) {
 }
 django.jQuery.ajaxSetup({
     beforeSend: function(xhr, settings) {
-        if (!csrfSafeMethod(settings.type) && !this.crossDomain) {
-            xhr.setRequestHeader("X-CSRFToken", csrftoken);
+        var token = csrftoken ? csrftoken : document.querySelector('[name=csrfmiddlewaretoken]').value;
+        if (!csrfSafeMethod(settings.type) && !this.crossDomain && token) {
+            xhr.setRequestHeader("X-CSRFToken", token);
         }
     }
 });
