@@ -16,7 +16,9 @@
 import binascii
 import re
 import shlex
+import sys
 import typing
+import warnings
 from collections import abc
 from datetime import datetime
 from datetime import timedelta
@@ -25,7 +27,6 @@ from ipaddress import ip_address
 from ipaddress import ip_network
 from typing import Any
 from typing import Dict
-from typing import Iterable
 from typing import List
 from typing import Optional
 from typing import Tuple
@@ -41,6 +42,7 @@ from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives import hashes
 from cryptography.hazmat.primitives.asymmetric import dsa
 from cryptography.hazmat.primitives.asymmetric import ec
+from cryptography.hazmat.primitives.asymmetric import ed448
 from cryptography.hazmat.primitives.asymmetric import ed25519
 from cryptography.hazmat.primitives.asymmetric import rsa
 from cryptography.hazmat.primitives.serialization import Encoding
@@ -48,11 +50,12 @@ from cryptography.x509.oid import NameOID
 
 from django.core.files.storage import get_storage_class
 from django.core.validators import URLValidator
-from django.utils.encoding import force_bytes
-from django.utils.encoding import force_str
+from django.utils import timezone as tz
 from django.utils.translation import gettext_lazy as _
 
 from . import ca_settings
+from .deprecation import RemovedInDjangoCA122Warning
+from .typehints import PRIVATE_KEY_TYPES
 from .typehints import Expires
 from .typehints import Literal
 from .typehints import ParsableGeneralName
@@ -60,45 +63,37 @@ from .typehints import ParsableGeneralNameList
 from .typehints import ParsableHash
 from .typehints import ParsableKeyCurve
 from .typehints import ParsableKeyType
-from .typehints import ParsableRelativeDistinguishedName
-from .typehints import PrivateKeyTypes
 from .typehints import SupportsIndex
 
 # List of possible subject fields, in order
 SUBJECT_FIELDS = [
-    "dnQualifier",
-    "C",
-    "postalCode",
-    "ST",
-    "L",
-    "DC",
-    "O",
-    "OU",
-    "title",
-    "CN",
-    "uid",
-    "emailAddress",
-    "serialNumber",
+    NameOID.DN_QUALIFIER,
+    NameOID.COUNTRY_NAME,
+    NameOID.POSTAL_CODE,
+    NameOID.STATE_OR_PROVINCE_NAME,
+    NameOID.LOCALITY_NAME,
+    NameOID.DOMAIN_COMPONENT,
+    NameOID.ORGANIZATION_NAME,
+    NameOID.ORGANIZATIONAL_UNIT_NAME,
+    NameOID.TITLE,
+    NameOID.COMMON_NAME,
+    NameOID.USER_ID,
+    NameOID.EMAIL_ADDRESS,
+    NameOID.SERIAL_NUMBER,
 ]
 
 # Description strings for various X509 extensions, taken from "man x509v3_config".
 EXTENDED_KEY_USAGE_DESC = _("Purposes for which the certificate public key can be used for.")
 KEY_USAGE_DESC = _("Permitted key usages.")
 
-#: Regular expression to match RDNs out of a full x509 name.
-NAME_RE = re.compile(
-    r"(?:/+|\A)\s*(?P<field>[^\s]*?)\s*"
-    r'=(?P<quote>[\'"])?\s*(?P<content>(?(quote).*?|[^/]*))\s*'
-    r"(?(quote)(?<!\\)(?P=quote))",
-    re.I,
-)
-
 #: Regular expression to match general names.
 GENERAL_NAME_RE = re.compile("^(email|URI|IP|DNS|RID|dirName|otherName):(.*)", flags=re.I)
 
 #: Regular expression matching certificate serials as hex
 SERIAL_RE = re.compile("^([0-9A-F][0-9A-F]:?)+[0-9A-F][0-9A-F]?$")
 
+UNSAFE_NAME_CHARS = re.compile(r'[\\/\'"]')
+
 SAN_NAME_MAPPINGS = {
     x509.DNSName: "DNS",
     x509.RFC822Name: "email",
@@ -116,7 +111,7 @@ OID_NAME_MAPPINGS: Dict[x509.ObjectIdentifier, str] = {
     NameOID.COMMON_NAME: "CN",
     NameOID.COUNTRY_NAME: "C",
     NameOID.DN_QUALIFIER: "dnQualifier",
-    NameOID.DOMAIN_COMPONENT: "DC",  # TODO: RFC 4519 also specifies "domainComponent"
+    NameOID.DOMAIN_COMPONENT: "DC",
     NameOID.EMAIL_ADDRESS: "emailAddress",  # not specified in RFC 4519
     NameOID.GENERATION_QUALIFIER: "generationQualifier",
     NameOID.GIVEN_NAME: "givenName",
@@ -134,18 +129,32 @@ OID_NAME_MAPPINGS: Dict[x509.ObjectIdentifier, str] = {
     NameOID.SERIAL_NUMBER: "serialNumber",
     NameOID.SNILS: "snils",  # undocumented
     NameOID.STATE_OR_PROVINCE_NAME: "ST",
-    NameOID.STREET_ADDRESS: "streetAddress",
+    NameOID.STREET_ADDRESS: "street",
     NameOID.SURNAME: "sn",
     NameOID.TITLE: "title",
     NameOID.UNSTRUCTURED_NAME: "unstructuredName",  # not specified in RFC 4519
-    NameOID.USER_ID: "uid",  # TODO: RFC 4519 also specifies "userid"
+    NameOID.USER_ID: "uid",
     NameOID.X500_UNIQUE_IDENTIFIER: "x500UniqueIdentifier",
 }
 
-# same, but reversed
+# same, but reversed, used for parsing
 NAME_OID_MAPPINGS = {v: k for k, v in OID_NAME_MAPPINGS.items()}
 
-# Some OIDs can occur multiple times
+# RFC 4519 adds some aliases so we add them here
+NAME_OID_MAPPINGS.update(
+    {
+        "commonName": NameOID.COMMON_NAME,
+        "domainComponent": NameOID.DOMAIN_COMPONENT,
+        "localityName": NameOID.LOCALITY_NAME,
+        "organizationName": NameOID.ORGANIZATION_NAME,
+        "organizationalUnitName": NameOID.ORGANIZATIONAL_UNIT_NAME,
+        "streetAddress": NameOID.STREET_ADDRESS,  # not specified in RFC 4519, but consistent with others
+        "surname": NameOID.SURNAME,
+        "userid": NameOID.USER_ID,
+    }
+)
+
+#: List of OIDs that may occur multiple times in a subject.
 MULTIPLE_OIDS = (
     NameOID.DOMAIN_COMPONENT,
     NameOID.ORGANIZATIONAL_UNIT_NAME,
@@ -153,7 +162,17 @@ MULTIPLE_OIDS = (
 )
 
 # uppercase values as keys for normalizing case
-NAME_CASE_MAPPINGS = {v.upper(): v for v in OID_NAME_MAPPINGS.values()}
+NAME_CASE_MAPPINGS = {k.upper(): v for k, v in NAME_OID_MAPPINGS.items()}
+
+ADMIN_SUBJECT_OIDS = (
+    NameOID.COUNTRY_NAME,
+    NameOID.STATE_OR_PROVINCE_NAME,
+    NameOID.LOCALITY_NAME,
+    NameOID.ORGANIZATION_NAME,
+    NameOID.ORGANIZATIONAL_UNIT_NAME,
+    NameOID.COMMON_NAME,
+    NameOID.EMAIL_ADDRESS,
+)
 
 
 #: Mapping of canonical hash algorithm names to the implementing classes
@@ -179,8 +198,7 @@ HASH_ALGORITHM_NAMES: typing.Dict[str, typing.Type[hashes.HashAlgorithm]] = {
     # hashes.SM3.name: hashes.SM3,
 }
 if hasattr(hashes, "SM3"):  # pragma: cryptography>=35.0 branch
-    # PYLINT NOTE: Remove pylint override once cryptography>=35.0 is used
-    HASH_ALGORITHM_NAMES[hashes.SM3.name] = hashes.SM3  # pylint: disable=no-member  # We check above
+    HASH_ALGORITHM_NAMES[hashes.SM3.name] = hashes.SM3
 
 #: Mapping of canonical elliptic curve names to the implementing classes
 ELLIPTIC_CURVE_NAMES: typing.Dict[str, typing.Type[ec.EllipticCurve]] = {
@@ -232,9 +250,19 @@ except ImportError:  # pragma: no cover
             return self
 
 
-def sort_name(subject: List[Tuple[str, str]]) -> List[Tuple[str, str]]:
+def make_naive(timestamp: datetime) -> datetime:
+    """Like :py:func:`~django.utils.timezone.make_naive`, but does not return an error if already naive."""
+    if tz.is_naive(timestamp) is False:
+        return tz.make_naive(timestamp)
+    return timestamp
+
+
+def sort_name(name: x509.Name) -> x509.Name:
     """Returns the subject in the correct order for a x509 subject."""
-    return sorted(subject, key=lambda e: SUBJECT_FIELDS.index(e[0]))
+    try:
+        return x509.Name(sorted(name, key=lambda attr: SUBJECT_FIELDS.index(attr.oid)))
+    except ValueError:
+        return name
 
 
 def encode_url(url: str) -> str:
@@ -279,42 +307,56 @@ def encode_dns(name: str) -> str:
     return idna.encode(name).decode("utf-8")
 
 
-def format_name(subject: Union[x509.Name, Iterable[Tuple[str, str]]]) -> str:
-    """Convert a subject into the canonical form for distinguished names.
+def format_name(subject: typing.Union[x509.Name, x509.RelativeDistinguishedName]) -> str:
+    """Convert a x509 name or relative name into the canonical form for distinguished names.
 
     This function does not take care of sorting the subject in any meaningful order.
 
+    .. deprecated:: 1.20.0
+
+       Passing a list of two-tuples is deprecated as of 1.20.0 and the functionality will be removed in
+       ``django_ca==1.22``.
+
     Examples::
 
-        >>> format_name([('CN', 'example.com'), ])
+        >>> format_name(x509.Name([x509.NameAttribute(NameOID.COMMON_NAME, 'example.com')]))
         '/CN=example.com'
-        >>> format_name([('CN', 'example.com'), ('O', "My Organization"), ])
-        '/CN=example.com/O=My Organization'
     """
-    if isinstance(subject, x509.Name):
-        subject = [(OID_NAME_MAPPINGS[s.oid], s.value) for s in subject]
 
-    values = "/".join([f"{k}={v}" for k, v in subject])
+    def _format_value(val: str) -> str:
+        # If val contains no unsafe chars, return it unchanged
+        if UNSAFE_NAME_CHARS.search(val) is None:
+            return val
+        return '"' + val.replace('"', r"\"").replace(r"\\", r"\\\\") + '"'
+
+    if isinstance(subject, (x509.Name, x509.RelativeDistinguishedName)):
+        items = [(OID_NAME_MAPPINGS[s.oid], s.value) for s in subject]
+    else:
+        warnings.warn(
+            "Passing a list to format_name() is deprecated, pass a str instead",
+            category=RemovedInDjangoCA122Warning,
+            stacklevel=1,
+        )
+        items = subject
+
+    values = "/".join([f"{k}={_format_value(v)}" for k, v in items])
     return f"/{values}"
 
 
-def format_relative_name(name: Union[x509.RelativeDistinguishedName, Iterable[Tuple[str, str]]]) -> str:
+def format_relative_name(name: x509.RelativeDistinguishedName) -> str:
     """Convert a relative name (RDN) into a canonical form.
 
-    Examples::
+    .. deprecated:: 1.20.0
 
-        >>> format_relative_name([('C', 'AT'), ('CN', 'example.com')])
-        '/C=AT/CN=example.com'
-        >>> format_relative_name(x509.RelativeDistinguishedName([
-        ...     x509.NameAttribute(NameOID.COMMON_NAME, 'example.com')
-        ... ]))
-        '/CN=example.com'
+       This function is deprecated in favor of ``format_name()``, which provides identical functionality. This
+       function will be removed in ``django_ca==1.22``.
     """
-    if isinstance(name, x509.RelativeDistinguishedName):
-        name = [(OID_NAME_MAPPINGS[s.oid], s.value) for s in name]
-
-    values = "/".join([f"{k}={v}" for k, v in name])
-    return f"/{values}"
+    warnings.warn(
+        "This function is deprecated, use format_name() instead.",
+        category=RemovedInDjangoCA122Warning,
+        stacklevel=1,
+    )
+    return format_name(name)
 
 
 def format_general_name(name: x509.GeneralName) -> str:
@@ -409,6 +451,30 @@ def hex_to_bytes(value: str) -> bytes:
     return binascii.unhexlify(value.replace(":", ""))
 
 
+def check_name(name: x509.Name) -> x509.Name:
+    """Check if `name` is a valid x509 Name.
+
+    This method raises ``ValueError`` if the CommonName contains an empty value or if any attribute not in
+    :py:attr:`~django_ca.utils.MULTIPLE_OIDS` occurs multiple times.
+
+    The method returns the name unchanged for convenience.
+    """
+    seen = set()
+
+    # for oid in set(oids):
+    for attr in name:
+        oid = attr.oid
+
+        # Check if any fields are duplicate where this is not allowed (e.g. multiple CommonName fields)
+        if oid in seen and oid not in MULTIPLE_OIDS:
+            raise ValueError(f'Subject contains multiple "{OID_NAME_MAPPINGS[attr.oid]}" fields')
+        seen.add(oid)
+
+        if oid == NameOID.COMMON_NAME and not attr.value:
+            raise ValueError("CommonName must not be an empty value")
+    return name
+
+
 def sanitize_serial(value: str) -> str:
     """Sanitize a serial provided by user/untrusted input.
 
@@ -430,67 +496,31 @@ def sanitize_serial(value: str) -> str:
     return serial
 
 
-def parse_csr(
-    csr: Union[x509.CertificateSigningRequest, str, bytes], csr_format: Encoding
-) -> x509.CertificateSigningRequest:
-    """Parse a CSR in the given format.
-
-    .. deprecated:: 1.18.0
-
-       This function is no longer useful and will be removed in django-ca 1.20.0.
-
-    Parameters
-    ----------
-
-    csr : str or bytes or :py:class:`~cg:cryptography.x509.CertificateSigningRequest`
-        The CSR to parse.
-    csr_format : :py:class:`~cg:cryptography.hazmat.primitives.serialization.Encoding`
-        The format that the CSR is in.
-    """
-
-    if isinstance(csr, x509.CertificateSigningRequest):  # pragma: no cover # not used since 1.18.0
-        return csr
-    if csr_format == Encoding.PEM:
-        return x509.load_pem_x509_csr(force_bytes(csr), default_backend())
-    if csr_format == Encoding.DER:
-        return x509.load_der_x509_csr(force_bytes(csr), default_backend())
-
-    raise ValueError(f"Unknown CSR format passed: {csr_format}")
-
-
-def parse_name(name: str) -> List[Tuple[str, str]]:
+def parse_name_x509(name: str) -> List[x509.NameAttribute]:
     """Parses a subject string as used in OpenSSLs command line utilities.
 
+    .. versionchanged:: 1.20.0
+
+       This function no longer returns the subject in pseudo-sorted order.
+
     The ``name`` is expected to be close to the subject format commonly used by OpenSSL, for example
     ``/C=AT/L=Vienna/CN=example.com/emailAddress=user@example.com``. The function does its best to be lenient
     on deviations from the format, object identifiers are case-insensitive (e.g. ``cn`` is the same as ``CN``,
     whitespace at the start and end is stripped and the subject does not have to start with a slash (``/``).
 
-    >>> parse_name('/CN=example.com')
-    [('CN', 'example.com')]
-    >>> parse_name('c=AT/l= Vienna/o="ex org"/CN=example.com')
-    [('C', 'AT'), ('L', 'Vienna'), ('O', 'ex org'), ('CN', 'example.com')]
+    >>> parse_name_x509('/CN=example.com')
+    [<NameAttribute(oid=<ObjectIdentifier(oid=2.5.4.3, name=commonName)>, value='example.com')>]
+    >>> parse_name_x509('c=AT/l= Vienna/o="quoting/works"/CN=example.com')  # doctest: +NORMALIZE_WHITESPACE
+    [<NameAttribute(oid=<ObjectIdentifier(oid=2.5.4.6, name=countryName)>, value='AT')>,
+     <NameAttribute(oid=<ObjectIdentifier(oid=2.5.4.7, name=localityName)>, value='Vienna')>,
+     <NameAttribute(oid=<ObjectIdentifier(oid=2.5.4.10, name=organizationName)>, value='quoting/works')>,
+     <NameAttribute(oid=<ObjectIdentifier(oid=2.5.4.3, name=commonName)>, value='example.com')>]
 
-    Dictionary keys are normalized to the values of :py:const:`OID_NAME_MAPPINGS` and keys will be sorted
-    based on x509 name specifications regardless of the given order:
+    The function also handles whitespace, quoting and slashes correctly:
 
-    >>> parse_name('L="Vienna / District"/EMAILaddress=user@example.com')
-    [('L', 'Vienna / District'), ('emailAddress', 'user@example.com')]
-    >>> parse_name('/C=AT/CN=example.com') == parse_name('/CN=example.com/C=AT')
-    True
-
-    Due to the magic of :py:const:`NAME_RE`, the function even supports quoting strings and including slashes,
-    so strings like ``/OU="Org / Org Unit"/CN=example.com`` will work as expected.
-
-    >>> parse_name('L="Vienna / District"/CN=example.com')
-    [('L', 'Vienna / District'), ('CN', 'example.com')]
-
-    But note that it's still easy to trick this function, if you really want to. The following example is
-    *not* a valid subject, the location is just bogus, and whatever you were expecting as output, it's
-    certainly different:
-
-    >>> parse_name('L="Vienna " District"/CN=example.com')
-    [('L', 'Vienna'), ('CN', 'example.com')]
+    >>> parse_name_x509('L="Vienna / District"/CN=example.com')  # doctest: +NORMALIZE_WHITESPACE
+    [<NameAttribute(oid=<ObjectIdentifier(oid=2.5.4.7, name=localityName)>, value='Vienna / District')>,
+     <NameAttribute(oid=<ObjectIdentifier(oid=2.5.4.3, name=commonName)>, value='example.com')>]
 
     Examples of where this string is used are:
 
@@ -500,55 +530,77 @@ def parse_name(name: str) -> List[Tuple[str, str]]:
         # openssl x509 -in cert.pem -noout -subject -nameopt compat
         /C=AT/L=Vienna/CN=example.com
     """
-    name = name.strip()
-    if not name:  # empty subjects are ok
-        return []
+    attrs = [t.split("=", 1) for t in split_str(name.strip(), "/")]
 
     try:
-        items = [(NAME_CASE_MAPPINGS[t[0].upper()], force_str(t[2])) for t in NAME_RE.findall(name)]
+        items = [(NAME_CASE_MAPPINGS[t[0].strip().upper()], t[1].strip()) for t in attrs]
     except KeyError as e:
         raise ValueError(f"Unknown x509 name field: {e.args[0]}") from e
 
-    # Check that no OIDs not in MULTIPLE_OIDS occur more then once
-    for key, oid in NAME_OID_MAPPINGS.items():
-        if sum(1 for t in items if t[0] == key) > 1 and oid not in MULTIPLE_OIDS:
-            raise ValueError(f'Subject contains multiple "{key}" fields')
-
-    return sort_name(items)
+    return [x509.NameAttribute(oid, value) for oid, value in items]
 
 
-def x509_name(name: Union[List[Tuple[str, str]], str]) -> x509.Name:
-    """Parses a subject into a :py:class:`x509.Name <cg:cryptography.x509.Name>`.
+def parse_name(name: str) -> List[Tuple[str, str]]:
+    """Parses a subject string as used in OpenSSLs command line utilities.
 
-    If ``name`` is a string, :py:func:`parse_name` is used to parse it.
+    .. deprecated:: 1.20.0
+
+       This function has been renamed to :py:func:`~django_ca.utils.parse_name_x509`. The old name will be
+       removed in ``django_ca==1.22``.
+    """
+
+    warnings.warn(
+        "parse_name() has been deprecated, use parse_name_x509() instead",
+        category=RemovedInDjangoCA122Warning,
+        stacklevel=1,
+    )
+    attrs = parse_name_x509(name)
+
+    # Parse OIDs back to their cannonical string representation
+    return [(OID_NAME_MAPPINGS[attr.oid], attr.value) for attr in attrs]
+
+
+def x509_name(name: str) -> x509.Name:
+    """Parses a string into a :py:class:`x509.Name <cg:cryptography.x509.Name>`.
+
+    .. deprecated:: 1.20.0
+
+       Passing a list of two-tuples is deprecated as of 1.20.0 and the functionality will be removed in
+       ``django_ca==1.22``.
 
     >>> x509_name('/C=AT/CN=example.com')
     <Name(C=AT,CN=example.com)>
-    >>> x509_name([('C', 'AT'), ('CN', 'example.com')])
-    <Name(C=AT,CN=example.com)>
     """
-    if isinstance(name, str):
-        name = parse_name(name)
+    if not isinstance(name, str):
+        warnings.warn(
+            "Passing a list to x509_name() is deprecated, pass a str instead",
+            category=RemovedInDjangoCA122Warning,
+            stacklevel=1,
+        )
+        return x509.Name([x509.NameAttribute(NAME_OID_MAPPINGS[typ], value) for typ, value in name])
 
-    return x509.Name([x509.NameAttribute(NAME_OID_MAPPINGS[typ], force_str(value)) for typ, value in name])
+    return check_name(x509.Name(parse_name_x509(name)))
 
 
-def x509_relative_name(name: ParsableRelativeDistinguishedName) -> x509.RelativeDistinguishedName:
+def x509_relative_name(name: str) -> x509.RelativeDistinguishedName:
     """Parse a relative name (RDN) into a :py:class:`~cg:cryptography.x509.RelativeDistinguishedName`.
 
     >>> x509_relative_name('/CN=example.com')
     <RelativeDistinguishedName(CN=example.com)>
-    >>> x509_relative_name([('CN', 'example.com')])
-    <RelativeDistinguishedName(CN=example.com)>
     """
-    if isinstance(name, x509.RelativeDistinguishedName):
-        return name
-    if isinstance(name, str):
-        name = parse_name(name)
 
-    return x509.RelativeDistinguishedName(
-        [x509.NameAttribute(NAME_OID_MAPPINGS[typ], force_str(value)) for typ, value in name]
-    )
+    msg = f"Passing a {name.__class__.__name__} to x509_relative_name() is deprecated, pass a str instead"
+
+    if isinstance(name, x509.RelativeDistinguishedName):
+        warnings.warn(msg, category=RemovedInDjangoCA122Warning, stacklevel=1)
+        return name
+    if isinstance(name, (list, tuple)):
+        warnings.warn(msg, category=RemovedInDjangoCA122Warning, stacklevel=1)
+        return x509.RelativeDistinguishedName(
+            [x509.NameAttribute(NAME_OID_MAPPINGS[typ], value) for typ, value in name]
+        )
+
+    return x509.RelativeDistinguishedName(parse_name_x509(name))
 
 
 def validate_email(addr: str) -> str:
@@ -621,107 +673,77 @@ def validate_hostname(hostname: str, allow_port: bool = False) -> str:
     return encoded
 
 
-@typing.overload
-def validate_key_parameters(
-    key_size: Optional[int], key_type: Literal["DSA"], ecc_curve: ParsableKeyCurve = None
-) -> Tuple[int, Literal["DSA"], None]:
-    ...
-
-
-@typing.overload
 def validate_key_parameters(
     key_size: Optional[int] = None,
-    key_type: Optional[Literal["RSA"]] = "RSA",
-    ecc_curve: ParsableKeyCurve = None,
-) -> Tuple[int, Literal["RSA"], None]:
-    ...
-
-
-@typing.overload
-def validate_key_parameters(
-    key_size: Optional[int], key_type: Literal["ECC"], ecc_curve: ParsableKeyCurve = None
-) -> Tuple[None, Literal["ECC"], ec.EllipticCurve]:
-    ...
-
-
-@typing.overload
-def validate_key_parameters(
-    key_size: Optional[int], key_type: Literal["EdDSA"], ecc_curve: ParsableKeyCurve = None
-) -> Tuple[None, Literal["EdDSA"], None]:
-    ...
-
-
-def validate_key_parameters(
-    key_size: Optional[int] = None,
-    key_type: Optional[ParsableKeyType] = "RSA",
-    ecc_curve: ParsableKeyCurve = None,
-) -> Tuple[Optional[int], Literal["RSA", "DSA", "ECC", "EdDSA"], Optional[ec.EllipticCurve]]:
+    key_type: ParsableKeyType = "RSA",
+    ecc_curve: typing.Optional[ec.EllipticCurve] = None,
+) -> None:
     """Validate parameters for private key generation and return sanitized values.
 
     This function can be used to fail early if invalid parameters are passed, before the private key is
     generated.
 
-    >>> validate_key_parameters()  # defaults
-    (1024, 'RSA', None)
-    >>> validate_key_parameters(4096, 'ECC', None)  # doctest: +ELLIPSIS
-    (None, 'ECC', <cryptography.hazmat.primitives.asymmetric.ec.SECP256R1 object at ...>)
+    >>> validate_key_parameters(4096, "RSA", None)
+    >>> validate_key_parameters(4096, "Ed448", None)  # Ed448 does not care about the key size
     >>> validate_key_parameters(4000, 'RSA', None)
     Traceback (most recent call last):
         ...
     ValueError: 4000: Key size must be a power of two
     """
-    if key_type is None:
-        key_type = "RSA"
 
-    if key_type == "ECC":
-        key_size = None
-        ecc_curve = parse_key_curve(ecc_curve)
-    elif key_type in ["DSA", "RSA"]:
-        ecc_curve = None
-        if key_size is None:
-            key_size = ca_settings.CA_DEFAULT_KEY_SIZE
+    if key_type not in ("RSA", "DSA", "ECC", "EdDSA", "Ed448"):
+        raise ValueError(f"{key_type}: Unknown key type")
 
-        if not is_power2(key_size):
+    if key_type in ("RSA", "DSA") and key_size is not None:
+        if is_power2(key_size) is False:
             raise ValueError(f"{key_size}: Key size must be a power of two")
         if key_size < ca_settings.CA_MIN_KEY_SIZE:
             raise ValueError(f"{key_size}: Key size must be least {ca_settings.CA_MIN_KEY_SIZE} bits")
-    elif key_type == "EdDSA":
-        key_size = ecc_curve = None
-    else:
-        raise ValueError(f"{key_type}: Unknown key type")
 
-    return key_size, key_type, ecc_curve
+    if key_type == "ECC" and ecc_curve is not None and not isinstance(ecc_curve, ec.EllipticCurve):
+        raise ValueError(f"{ecc_curve}: Must be a subclass of ec.EllipticCurve")
 
 
 @typing.overload
-def generate_private_key(key_size: int, key_type: Literal["DSA"], ecc_curve: None) -> dsa.DSAPrivateKey:
-    ...
-
-
-@typing.overload
-def generate_private_key(key_size: int, key_type: Literal["RSA"], ecc_curve: None) -> rsa.RSAPrivateKey:
+def generate_private_key(
+    key_size: typing.Optional[int], key_type: Literal["DSA"], ecc_curve: typing.Optional[ec.EllipticCurve]
+) -> dsa.DSAPrivateKey:
     ...
 
 
 @typing.overload
 def generate_private_key(
-    key_size: None, key_type: Literal["ECC"], ecc_curve: ec.EllipticCurve
+    key_size: typing.Optional[int], key_type: Literal["RSA"], ecc_curve: typing.Optional[ec.EllipticCurve]
+) -> rsa.RSAPrivateKey:
+    ...
+
+
+@typing.overload
+def generate_private_key(
+    key_size: typing.Optional[int], key_type: Literal["ECC"], ecc_curve: typing.Optional[ec.EllipticCurve]
 ) -> ec.EllipticCurvePrivateKey:
     ...
 
 
 @typing.overload
 def generate_private_key(
-    key_size: int, key_type: Literal["EdDSA"], ecc_curve: None
+    key_size: typing.Optional[int], key_type: Literal["EdDSA"], ecc_curve: typing.Optional[ec.EllipticCurve]
 ) -> ed25519.Ed25519PrivateKey:
     ...
 
 
+@typing.overload
+def generate_private_key(
+    key_size: typing.Optional[int], key_type: Literal["Ed448"], ecc_curve: typing.Optional[ec.EllipticCurve]
+) -> ed448.Ed448PrivateKey:
+    ...
+
+
 def generate_private_key(
     key_size: Optional[int],
-    key_type: Literal["RSA", "DSA", "ECC", "EdDSA"],
+    key_type: ParsableKeyType,
     ecc_curve: Optional[ec.EllipticCurve],
-) -> PrivateKeyTypes:
+) -> PRIVATE_KEY_TYPES:
     """Generate a private key.
 
     This function assumes that you called :py:func:`~django_ca.utils.validate_key_parameters` on the input
@@ -731,11 +753,12 @@ def generate_private_key(
     ----------
 
     key_size : int
-        The size of the private key (not used for ECC keys).
-    key_type : {'RSA', 'DSA', 'ECC'}
+        The size of the private key. The value is  ignored if ``key_type`` is not ``"DSA"`` or ``"RSA"``.
+    key_type : {'RSA', 'DSA', 'ECC', 'EdDSA', 'Ed448'}
         The type of the private key.
     ecc_curve : :py:class:`~cg:cryptography.hazmat.primitives.asymmetric.ec.EllipticCurve`
-        The ECC curve to use for an ECC key.
+        An elliptic curve to use for ECC keys. This parameter is ignored if ``key_type`` is not ``"ECC"``.
+        Defaults to the :ref:`CA_DEFAULT_ECC_CURVE <settings-ca-default-ecc-curve>`.
 
     Returns
     -------
@@ -743,16 +766,32 @@ def generate_private_key(
     key
         A private key of the appropriate type.
     """
-    if key_type == "DSA" and key_size is not None:
+    # Make sure that parameters are valid
+    validate_key_parameters(key_size, key_type, ecc_curve)
+
+    if key_type == "DSA":
+        if key_size is None:
+            key_size = ca_settings.CA_DEFAULT_KEY_SIZE
+
         return dsa.generate_private_key(key_size=key_size, backend=default_backend())
-    if key_type == "ECC" and ecc_curve is not None:
+    if key_type == "ECC":
+        if ecc_curve is None:
+            ecc_curve = ca_settings.CA_DEFAULT_ECC_CURVE()
+
         return ec.generate_private_key(ecc_curve, default_backend())
-    if key_type == "EdDSA" and ecc_curve is None and key_size is None:
+    if key_type == "EdDSA":
         return ed25519.Ed25519PrivateKey.generate()
-    if key_type == "RSA" and key_size is not None:
+    if key_type == "Ed448":
+        return ed448.Ed448PrivateKey.generate()
+    if key_type == "RSA":
+        if key_size is None:
+            key_size = ca_settings.CA_DEFAULT_KEY_SIZE
+
         return rsa.generate_private_key(public_exponent=65537, key_size=key_size, backend=default_backend())
 
-    raise ValueError(f"{key_type}: Invalid key type.")
+    # COVERAGE NOTE: Unreachable code, as all possible key_types are handled above and validate_key_parameters
+    #                 would raise for any other key types.
+    raise ValueError(f"{key_type}: Invalid key type.")  # pragma: no cover
 
 
 def parse_general_name(name: ParsableGeneralName) -> x509.GeneralName:
@@ -1097,14 +1136,19 @@ def parse_key_curve(value: ParsableKeyCurve = None) -> ec.EllipticCurve:
     if isinstance(value, ec.EllipticCurve):
         return value  # name was already parsed
     if value is None:
-        return ca_settings.CA_DEFAULT_ECC_CURVE
+        return ca_settings.CA_DEFAULT_ECC_CURVE()
 
     if value in ELLIPTIC_CURVE_NAMES:
         return ELLIPTIC_CURVE_NAMES[value]()
 
-    curve: Type[ec.EllipticCurve] = getattr(ec, value.strip(), type)
+    error_msg = f"{value}: Not a known Eliptic Curve"
+
+    try:
+        curve: Type[ec.EllipticCurve] = getattr(ec, value.strip())
+    except AttributeError as ex:
+        raise ValueError(error_msg) from ex
     if not issubclass(curve, ec.EllipticCurve):
-        raise ValueError(f"{value}: Not a known Eliptic Curve")
+        raise ValueError(error_msg)
     return curve()
 
 
@@ -1126,6 +1170,7 @@ def get_cert_builder(expires: datetime, serial: Optional[int] = None) -> x509.Ce
     if serial is None:
         serial = x509.random_serial_number()
 
+    expires = make_naive(expires)
     if expires <= now:
         raise ValueError("expires must be in the future")
 
@@ -1164,24 +1209,58 @@ def read_file(path: str) -> bytes:
 #        fh.write(data)
 
 
-def shlex_split(val: str, sep: str) -> List[str]:
+def split_str(val: str, sep: str) -> typing.Iterator[str]:
     """Split a character on the given set of characters.
 
     Example::
 
-        >>> shlex_split('foo,bar', ', ')
+        >>> list(split_str('foo,bar', ', '))
         ['foo', 'bar']
-        >>> shlex_split('foo\\\\,bar1', ',')  # escape a separator
+        >>> list(split_str('foo\\\\,bar1', ','))  # escape a separator
         ['foo,bar1']
-        >>> shlex_split('"foo,bar", bla', ', ')
-        ['foo,bar', 'bla']
-        >>> shlex_split('foo,"bar,bla"', ',')
+        >>> list(split_str('foo,"bar,bla"', ','))  # do not split on quoted separator
         ['foo', 'bar,bla']
+
+    Note that `sep` gives one or more separator characters, not a single separator string::
+
+        >>> list(split_str("foo,bar bla", ", "))
+        ['foo', 'bar', 'bla']
+
+    Unlike ``str.split()``, separators at the start/end of a string are simply ignored, as are multiple
+    subsequent separators::
+
+        >>> list(split_str("/C=AT//ST=Vienna///OU=something//CN=example.com/", "/"))
+        ['C=AT', 'ST=Vienna', 'OU=something', 'CN=example.com']
+
+    Parameters
+    ----------
+
+    val : str
+        The string to split.
+    sep: str
+        String of characters that are considered separators.
     """
     lex = shlex.shlex(val, posix=True)
+    lex.commenters = ""
     lex.whitespace = sep
     lex.whitespace_split = True
-    return list(lex)
+    yield from lex
+
+
+def shlex_split(val: str, sep: str) -> List[str]:
+    """Split a character on the given set of characters.
+
+    .. deprecated:: 1.20.0
+
+       This function has been renamed to :py:func:`~django_ca.utils.split_str`. The old name will be removed
+       in ``django_ca==1.22``.
+    """
+    warnings.warn(
+        "shlex_split() has been deprecated, use split_str() instead",
+        category=RemovedInDjangoCA122Warning,
+        stacklevel=2,
+    )
+    return list(split_str(val=val, sep=sep))
 
 
 class GeneralNameList(List[x509.GeneralName]):
@@ -1285,11 +1364,13 @@ class GeneralNameList(List[x509.GeneralName]):
         """Equivalent to list.extend()."""
         list.extend(self, (parse_general_name(i) for i in iterable))
 
-    def index(self, value: ParsableGeneralName, *args: int) -> int:
+    def index(
+        self, value: ParsableGeneralName, start: SupportsIndex = 0, stop: SupportsIndex = sys.maxsize
+    ) -> int:
         """Equivalent to list.index()."""
-        return list.index(self, parse_general_name(value), *args)
+        return list.index(self, parse_general_name(value), start, stop)
 
-    def insert(self, index: int, value: ParsableGeneralName) -> None:
+    def insert(self, index: SupportsIndex, value: ParsableGeneralName) -> None:
         """Equivalent to list.insert()."""
         list.insert(self, index, parse_general_name(value))
 
@@ -1311,4 +1392,4 @@ def get_crl_cache_key(
 # NOTE: get_storage_class is typed to Storage (but really returns the subclass FileSystemStorage).
 #       The default kwargs trigger a type error because the default works for the subclass.
 ca_storage_cls = get_storage_class(ca_settings.CA_FILE_STORAGE)
-ca_storage = ca_storage_cls(**ca_settings.CA_FILE_STORAGE_KWARGS)  # type: ignore
+ca_storage = ca_storage_cls(**ca_settings.CA_FILE_STORAGE_KWARGS)
